import time
import uuid

class TraceContext:
  def __init__(self, manager, name, input=None):
    self.manager = manager
    self.name = name
    self.input = input
    self.output = None
    self.trace_id = str(uuid.uuid4())
    self.start_time = None

  def __enter__(self):
    self.start_time = time.time()
    return self

  def __exit__(self, exc_type, exc, tb):
    end_time = time.time()

    self.manager.push_all(
      trace_id=self.trace_id,
      name=self.name,
      input=self.input,
      output=self.output,
      start_time=self.start_time,
      end_time=end_time,
      error=str(exc) if exc else None
    )


def get_env(key):

  defaults = {
    "LANGFUSE_SECRET_KEY": "sk-lf-ccea8a1f-d572-4dcd-af4c-d73e1dbc2eb3",
    "LANGFUSE_PUBLIC_KEY": "pk-lf-a9178954-2e7f-43cc-9045-21d544f12231",
    "LANGFUSE_HOST": "https://us.cloud.langfuse.com",

    "LANGCHAIN_TRACING_V2": "true",
    "LANGCHAIN_API_KEY": "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",


    "SENTRY_DSN": "https://f515b78b435ab35b5e068e36c6f87fed@o4511019423105024.ingest.us.sentry.io/4511075166257152",
    "PHOENIX_ENDPOINT": "http://127.0.0.1:6006/v1/traces"
  }

  return defaults.get(key)


class ToolManager:
  def __init__(self):
    self.tools = []

  def register(self, tool):
    self.tools.append(tool)

  def push_all(self, **kwargs):
    for tool in self.tools:
      try:
        tool.push(**kwargs)
      except Exception as e:
        print(f"[OBS ERROR] {tool.__class__.__name__}: {e}")


import sentry_sdk
from .base import BaseTool
from obs.utils.config import get_env

class SentryTool(BaseTool):
  def init(self):
    sentry_sdk.init(dsn=get_env("SENTRY_DSN"))

  def push(self, trace_id, name, input, output, start_time, end_time, error):
    if error:
      sentry_sdk.capture_exception(Exception(error))


from obs.tools.base import BaseTool
from opentelemetry import trace


class LangfuseTool(BaseTool):

  def init(self):
    self.tracer = trace.get_tracer("langfuse")

  def push(self, trace_id, name, input, output, start_time, end_time, error):

    with self.tracer.start_as_current_span(name) as span:
      span.set_attribute("input", str(input))
      span.set_attribute("output", str(output))
      span.set_attribute("trace_id", trace_id)

      if error:
        span.set_attribute("error", error)


from opentelemetry.sdk.trace import TracerProvider
from opentelemetry import trace
from opentelemetry.sdk.trace.export import BatchSpanProcessor
from opentelemetry.exporter.otlp.proto.http.trace_exporter import OTLPSpanExporter

from obs.utils.config import get_env
import base64

_initialized = False

def init_tracer():

  global _initialized

  if _initialized:
    return trace.get_tracer_provider()

  provider = TracerProvider()
  trace.set_tracer_provider(provider)

  host = get_env("LANGFUSE_HOST")
  pk = get_env("LANGFUSE_PUBLIC_KEY")
  sk = get_env("LANGFUSE_SECRET_KEY")

  creds = f"{pk}:{sk}"
  encoded = base64.b64encode(creds.encode()).decode()

  headers = {
    "Authorization": f"Basic {encoded}"
  }

  langfuse_exporter = OTLPSpanExporter(
    endpoint=f"{host}/api/public/otel/v1/traces",
    headers=headers
  )

  provider.add_span_processor(BatchSpanProcessor(langfuse_exporter))

  phoenix_exporter = OTLPSpanExporter(
    endpoint=get_env("PHOENIX_ENDPOINT")
  )

  provider.add_span_processor(BatchSpanProcessor(phoenix_exporter))

  print("🚀 OBS SDK INITIALIZED")

  _initialized = True

  return provider


from contextlib import contextmanager
import time
import uuid

class Tool:

  def init(self, tools=None):

    from obs.utils.otel import init_tracer
    from obs.tools import (
      LangfuseTool,
      PhoenixTool,
      LangSmithTool,
      SentryTool
    )

    init_tracer()

    if tools is None:
      tools = [
        LangfuseTool(),
        PhoenixTool(),
        LangSmithTool(),
        SentryTool()
      ]

    self.tools = tools

    for t in self.tools:
      try:
        t.init()
      except Exception as e:
        print(f"[OBS INIT ERROR] {t.__class__.__name__}: {e}")


  @contextmanager
  def trace(self, name, input=None):

    trace_id = str(uuid.uuid4())
    start_time = time.time()
    error = None

    class TraceObj:
      output = None

    t = TraceObj()

    try:
      yield t
    except Exception as e:
      error = str(e)
      raise e
    finally:
      end_time = time.time()

      for tool in self.tools:
        try:
          tool.push(
            trace_id=trace_id,
            name=name,
            input=input,
            output=t.output,
            start_time=start_time,
            end_time=end_time,
            error=error
          )
        except Exception as e:
          print(f"[OBS ERROR] {tool.__class__.__name__}: {e}")


import os
from langsmith import Client
from .base import BaseTool
from obs.utils.config import get_env


class LangSmithTool(BaseTool):

  def init(self):

    os.environ["LANGCHAIN_TRACING_V2"] = get_env("LANGCHAIN_TRACING_V2")
    os.environ["LANGCHAIN_API_KEY"] = get_env("LANGCHAIN_API_KEY")

    self.client = Client()


  def push(self, trace_id, name, input, output, start_time, end_time, error):

    self.client.create_run(
      name=name,
      inputs={"input": input},
      outputs={"output": output},
      error=error,
      run_type="chain"
    )